@@ -27,6 +27,8 @@ message () {
   echo " -m memory.tmx    use a translation memory to recycle translations"
   echo " -o direction     translation direction using the translation memory,"
   echo "                  by default 'direction' is used instead"
+  echo " -s               allow translation memory segments to match before space (as well as"
+  echo "                  before punctuation)"
   echo " -l               lists the available translation directions and exits"
   echo " -V               print Apertium version"
   # This is wrong and should be deprecated, but until all tools consider \0 flush without -z then this helps to have
@@ -155,7 +157,11 @@ run_mode_wblank () {
 
 run_mode_wblank_force_flush () {
     NULL_FLUSH=(-z)
-    run_mode_wblank
+    if [[ -z "$TRANSLATION_MEMORY_FILE" ]]; then
+      cat
+    else
+      lt-tmxproc "${NULL_FLUSH[@]}" "${TMXPROC_OPTIONS[@]}" "$TMCOMPFILE"
+    fi | run_mode_wblank
 }
 
 test_zip () {
@@ -193,7 +199,7 @@ translate_latex() {
     apertium-deslatex "${FORMAT_OPTIONS[@]}" | \
     if [[ -z "$TRANSLATION_MEMORY_FILE" ]];
     then cat;
-    else lt-tmxproc "$TMCOMPFILE";
+    else lt-tmxproc "${TMXPROC_OPTIONS[@]}" "$TMCOMPFILE";
     fi | \
       run_mode_wblank | \
       apertium-relatex| \
@@ -216,7 +222,7 @@ translate_latex_raw() {
 translate_odt () {
   check_transfuse
   if [[ $USE_TRANSFUSE = "yes" ]]; then
-    tf-extract -f odt "$INFILE" | run_mode_wblank_force_flush | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
+    tf-extract "${TF_EXTRACT_OPTIONS[@]}" -f odt "$INFILE" | run_mode_wblank_force_flush | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
     return
   fi
 
@@ -237,7 +243,7 @@ translate_odt () {
   apertium-desodt "${FORMAT_OPTIONS[@]}" |\
   if [[ -z "$TRANSLATION_MEMORY_FILE" ]];
   then cat;
-  else lt-tmxproc "$TMCOMPFILE";
+  else lt-tmxproc "${TMXPROC_OPTIONS[@]}" "$TMCOMPFILE";
   fi | \
     run_mode_wblank | \
     apertium-reodt|\
@@ -260,7 +266,7 @@ translate_odt () {
 translate_docx () {
   check_transfuse
   if [[ $USE_TRANSFUSE = "yes" ]]; then
-    tf-extract -f docx "$INFILE" | run_mode_wblank_force_flush | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
+    tf-extract "${TF_EXTRACT_OPTIONS[@]}" -f docx "$INFILE" | run_mode_wblank_force_flush | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
     return
   fi
 
@@ -290,12 +296,12 @@ translate_docx () {
   done;
 
   find "$INPUT_TMPDIR" -name "*.xml" |\
-  grep -E -v -i '(settings|theme|styles|font|rels|docProps|Content_Types)' |\
+  grep -E -v -i '(settings|theme|styles|font|rels|docProps|Content_Types|customXml)' |\
   apertium-adapt-docx -n |\
   apertium-deswxml "${FORMAT_OPTIONS[@]}" |\
   if [[ -z "$TRANSLATION_MEMORY_FILE" ]];
   then cat;
-  else lt-tmxproc "$TMCOMPFILE";
+  else lt-tmxproc "${TMXPROC_OPTIONS[@]}" "$TMCOMPFILE";
   fi | \
     run_mode_wblank | \
     apertium-rewxml|\
@@ -317,7 +323,7 @@ translate_docx () {
 translate_pptx () {
   check_transfuse
   if [[ $USE_TRANSFUSE = "yes" ]]; then
-    tf-extract -f pptx "$INFILE" | run_mode_wblank_force_flush | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
+    tf-extract "${TF_EXTRACT_OPTIONS[@]}" -f pptx "$INFILE" | run_mode_wblank_force_flush | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
     return
   fi
 
@@ -351,7 +357,7 @@ translate_pptx () {
   apertium-despptx "${FORMAT_OPTIONS[@]}" |\
   if [[ -z "$TRANSLATION_MEMORY_FILE" ]];
   then cat;
-  else lt-tmxproc "$TMCOMPFILE";
+  else lt-tmxproc "${TMXPROC_OPTIONS[@]}" "$TMCOMPFILE";
   fi | \
     run_mode_wblank | \
     apertium-repptx |\
@@ -389,7 +395,7 @@ translate_xlsx () {
   apertium-desxlsx "${FORMAT_OPTIONS[@]}" |\
   if [[ -z "$TRANSLATION_MEMORY_FILE" ]];
   then cat;
-  else lt-tmxproc "$TMCOMPFILE";
+  else lt-tmxproc "${TMXPROC_OPTIONS[@]}" "$TMCOMPFILE";
   fi | \
     run_mode_wblank | \
     apertium-rexlsx |\
@@ -411,28 +417,28 @@ translate_xlsx () {
 translate_html () {
   check_transfuse
   if [[ $USE_TRANSFUSE = "yes" ]]; then
-    tf-extract -f html "$INFILE" | run_mode_wblank_force_flush | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
+    tf-extract "${TF_EXTRACT_OPTIONS[@]}" -f html "$INFILE" | run_mode_wblank_force_flush | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
     return
   fi
 
   apertium-deshtml "${FORMAT_OPTIONS[@]}" "$INFILE" | \
     if [[ -z "$TRANSLATION_MEMORY_FILE" ]]; then
       cat
-    else lt-tmxproc "$TMCOMPFILE";
+    else lt-tmxproc "${TMXPROC_OPTIONS[@]}" "$TMCOMPFILE";
     fi | run_mode_wblank | if [[ -z "$REDIR" ]]; then apertium-rehtml; else apertium-rehtml > "$OUT_FILE"; fi
 }
 
 translate_htmlnoent () {
   check_transfuse
   if [[ $USE_TRANSFUSE = "yes" ]]; then
-    tf-extract -f html "$INFILE" | run_mode_wblank_force_flush | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
+    tf-extract "${TF_EXTRACT_OPTIONS[@]}" -f html "$INFILE" | run_mode_wblank_force_flush | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
     return
   fi
 
   apertium-deshtml "${FORMAT_OPTIONS[@]}" "$INFILE" | \
     if [[ -z "$TRANSLATION_MEMORY_FILE" ]]; then
       cat
-    else lt-tmxproc "$TMCOMPFILE";
+    else lt-tmxproc "${TMXPROC_OPTIONS[@]}" "$TMCOMPFILE";
     fi | run_mode_wblank | if [[ -z "$REDIR" ]]; then apertium-rehtml-noent; else apertium-rehtml-noent > "$OUT_FILE"; fi
 }
 
@@ -440,14 +446,14 @@ translate_htmlalt () {
   apertium-deshtml-alt "${FORMAT_OPTIONS[@]}" "$INFILE" | \
     if [[ -z "$TRANSLATION_MEMORY_FILE" ]]; then
       cat
-    else lt-tmxproc "$TMCOMPFILE";
+    else lt-tmxproc "${TMXPROC_OPTIONS[@]}" "$TMCOMPFILE";
     fi | run_mode_wblank | if [[ -z "$REDIR" ]]; then apertium-rehtml-noent; else apertium-rehtml-noent > "$OUT_FILE"; fi
 }
 
 translate_line () {
   check_transfuse
   if [[ $USE_TRANSFUSE = "yes" ]]; then
-	tf-extract -f line "$INFILE" | run_mode_wblank_force_flush | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
+	tf-extract "${TF_EXTRACT_OPTIONS[@]}" -f line "$INFILE" | run_mode_wblank_force_flush | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
 	return
   fi
   # TODO: lt-proc inserts spaces before parts of mwe's that cross
@@ -457,7 +463,7 @@ translate_line () {
       if [[ -z "$TRANSLATION_MEMORY_FILE" ]]; then
           cat
       else
-        lt-tmxproc "$TMCOMPFILE";
+        lt-tmxproc "${TMXPROC_OPTIONS[@]}" "$TMCOMPFILE";
       fi | \
           run_mode_wblank | \
               if [[ -z "$REDIR" ]]; then
@@ -486,7 +492,9 @@ FORMAT="txt"
 DATADIR=$APERTIUM_DATADIR
 TRANSLATION_MEMORY_DIRECTION=$PAIR
 LIST_MODES_AND_EXIT=false
+TMXPROC_OPTIONS=()
 FORMAT_OPTIONS=()
+TF_EXTRACT_OPTIONS=()
 NULL_FLUSH=()
 
 # Skip (but store) non-option arguments that come before options:
@@ -502,18 +510,19 @@ while [[ $OPTIND -le $# ]]; do
 done
 
 
-while getopts ":uahlVzf:d:m:o:nH" opt; do
+while getopts ":uahlVzf:d:m:o:snH" opt; do
   case "$opt" in
     f) FORMAT=$OPTARG ;;
     d) DATADIR=$OPTARG ;;
     m) TRANSLATION_MEMORY_FILE=$OPTARG ;;
     o) TRANSLATION_MEMORY_DIRECTION=$OPTARG ;;
+    s) TMXPROC_OPTIONS+=(-s) ;;
     u) UWORDS="no" ;;
-    n) FORMAT_OPTIONS+=(-n) ;;
-    H) FORMAT_OPTIONS+=(-o) ;;
+    n) FORMAT_OPTIONS+=(-n); TF_EXTRACT_OPTIONS+=(--apertium-n) ;;
+    H) FORMAT_OPTIONS+=(-o); TF_EXTRACT_OPTIONS+=(-H) ;;
     a) OPTION_TAGGER="-m" ;;
     l) LIST_MODES_AND_EXIT=true ;;
-	V) echo "${apertium_version}" && exit 0 ;;
+    V) echo "${apertium_version}" && exit 0 ;;
     z) NULL_FLUSH+=(-z) ;;
     h) message ;;
     \?) echo "ERROR: Unknown option $OPTARG" >&2; message >&2 ;;
@@ -557,7 +566,7 @@ fi
 if [[ -n $TRANSLATION_MEMORY_FILE ]]; then
   if ! lt-tmxcomp "$TRANSLATION_MEMORY_DIRECTION" "$TRANSLATION_MEMORY_FILE" "$TMCOMPFILE" >/dev/null; then
     echo "Error: Cannot compile TM '$TRANSLATION_MEMORY_FILE'" >&2
-    echo"   hint: use -o parameter" >&2
+    echo "   hint: use -o parameter to specify translation memory source/target languages" >&2
     message >&2
   fi
 fi
@@ -748,7 +757,7 @@ fi | if [[ -z "$TRANSLATION_MEMORY_FILE" ]];
   then
     cat
   else
-    lt-tmxproc "$TMCOMPFILE"
+    lt-tmxproc "${TMXPROC_OPTIONS[@]}" "$TMCOMPFILE"
   fi | run_mode_wblank | if [[ "$FORMAT" = "none" ]]; then
     if [[ -z "$REDIR" ]]; then
       cat
