@@ -52,7 +52,7 @@ unformat_latex()
   test_gawk
 
   if [ "$FICHERO" = "" ]
-  then FICHERO=$(mktemp $TMPDIR/apertium.XXXXXXXX)
+  then FICHERO=$(mktemp "$TMPDIR"/apertium.XXXXXXXX)
        cat > "$FICHERO"
        BORRAFICHERO="true"
   fi
@@ -69,7 +69,7 @@ unformat_latex()
 
 unformat_odt ()
 {
-  INPUT_TMPDIR=$(mktemp -d $TMPDIR/apertium.XXXXXXXX)
+  INPUT_TMPDIR=$(mktemp -d "$TMPDIR"/apertium.XXXXXXXX)
 
   locale_utf8
   test_zip
@@ -83,7 +83,7 @@ unformat_odt ()
 
 unformat_docx ()
 {
-  INPUT_TMPDIR=$(mktemp -d $TMPDIR/apertium.XXXXXXXX)
+  INPUT_TMPDIR=$(mktemp -d "$TMPDIR"/apertium.XXXXXXXX)
 
   locale_utf8
   test_zip
@@ -91,7 +91,7 @@ unformat_docx ()
   unzip -q -o -d "$INPUT_TMPDIR" "$FICHERO"
 
   for i in $(find "$INPUT_TMPDIR"|grep "xlsx$");
-  do LOCALTEMP=$(mktemp $TMPDIR/apertium.XXXXXXXX)
+  do LOCALTEMP=$(mktemp "$TMPDIR"/apertium.XXXXXXXX)
      apertium -f xlsx -d "$DIRECTORY" "$OPCIONU" "$PREFIJO" <"$i" >"$LOCALTEMP";
      cp "$LOCALTEMP" "$i";
      rm "$LOCALTEMP";
@@ -106,7 +106,7 @@ unformat_docx ()
 
 unformat_pptx ()
 {
-  INPUT_TMPDIR=$(mktemp -d $TMPDIR/apertium.XXXXXXXX)
+  INPUT_TMPDIR=$(mktemp -d "$TMPDIR"/apertium.XXXXXXXX)
 
   locale_utf8
   test_zip
@@ -114,7 +114,7 @@ unformat_pptx ()
   unzip -q -o -d "$INPUT_TMPDIR" "$FICHERO"
 
   for i in $(find "$INPUT_TMPDIR"|grep "xlsx$");
-  do LOCALTEMP=$(mktemp $TMPDIR/apertium.XXXXXXXX)
+  do LOCALTEMP=$(mktemp "$TMPDIR"/apertium.XXXXXXXX)
      apertium -f xlsx -d "$DIRECTORY" "$OPCIONU" "$PREFIJO" <"$i" >"$LOCALTEMP"
      cp "$LOCALTEMP" "$i"
      rm "$LOCALTEMP"
@@ -129,7 +129,7 @@ unformat_pptx ()
 
 unformat_xlsx ()
 {
-  INPUT_TMPDIR=$(mktemp -d $TMPDIR/apertium.XXXXXXXX)
+  INPUT_TMPDIR=$(mktemp -d "$TMPDIR"/apertium.XXXXXXXX)
 
   locale_utf8
   test_zip
@@ -226,4 +226,4 @@ case "$FORMATADOR" in
 
 esac
 
-apertium-des$FORMATADOR "$FICHERO" >"$SALIDA"
+apertium-des"$FORMATADOR" "$FICHERO" >"$SALIDA"
