@@ -18,6 +18,11 @@
 #include <sstream>
 #include <string>
 
+namespace {
+// ToDo: ABI break to change member what_ back to std::string
+std::string res;
+}
+
 namespace Apertium {
 ExceptionType::ExceptionType(const char *const what_)
   : what_(to_ustring(what_)) {}
@@ -38,7 +43,7 @@ ExceptionType::~ExceptionType() throw() {}
 
 const char *ExceptionType::what() const throw()
 {
-  std::string res;
+  res.clear();
   utf8::utf16to8(what_.begin(), what_.end(), std::back_inserter(res));
   return res.c_str();
 }
