@@ -1,9 +1,9 @@
 if [ $# != 2 ]
 then if [ $# != 3 ]
-     then echo "USAGE: $(basename $0) [-O] <input_file> <output_file>";
+     then echo "USAGE: $(basename "$0") [-O] <input_file> <output_file>";
           exit 1;
-     elif [ $1 != "-O" ]
-     then echo "USAGE: $(basename $0) [-O] <input file> <output_file>";
+     elif [ "$1" != "-O" ]
+     then echo "USAGE: $(basename "$0") [-O] <input file> <output_file>";
           exit 1;
      fi
 fi
@@ -13,14 +13,14 @@ FILE1=$1;
 FILE2=$2;
 
 if [ $# = 2 ]
-then if [ ! -e $1 ] 
+then if [ ! -e "$1" ]
      then echo "ERROR: '$1' file not found";
           exit 1;
-     fi 
+     fi
 fi
 
 if [ $# = 3 ]
-then if [ ! -e $2 ]
+then if [ ! -e "$2" ]
      then echo "ERROR: '$2' file not found";
           exit 1;
      fi
