@@ -24,7 +24,6 @@
 using namespace std;
 
 Postchunk::Postchunk()
-  : word(0), in_wblank(false), inword(false)
 {}
 
 bool
@@ -349,6 +348,7 @@ Postchunk::processLet(xmlNode *localroot)
 void
 Postchunk::processModifyCase(xmlNode *localroot)
 {
+  if (dictionary_case) return;
   xmlNode *leftSide = NULL, *rightSide = NULL;
 
   for (auto i : children(localroot)) {
@@ -567,24 +567,39 @@ Postchunk::postchunk(InputFile& in, UFILE* out)
     {
       if(lastrule != NULL)
       {
-        int words_to_consume = applyRule();
-        if (words_to_consume == -1) {
+        int num_words_to_consume = applyRule();
+
+        //Consume all the words from the input which matched the rule.
+        //This piece of code is executed unless the rule contains a "reject-current-rule" instruction
+        if(num_words_to_consume < 0)
+        {
           banned_rules.clear();
           input_buffer.setPos(last);
-        } else if (words_to_consume == 1) {
+        }
+        else if(num_words_to_consume > 0)
+        {
           banned_rules.clear();
-          if (prev_last >= input_buffer.getSize()) {
+          if(prev_last >= input_buffer.getSize())
+          {
             input_buffer.setPos(0);
-          } else {
+          }
+          else
+          {
             input_buffer.setPos(prev_last+1);
           }
-          while (true) {
-            TransferToken& tt = input_buffer.next();
-            if (tt.getType() == tt_word) {
-              break;
+          int num_consumed_words = 0;
+          while(num_consumed_words < num_words_to_consume && !input_buffer.isEmpty())
+          {
+            TransferToken& local_tt = input_buffer.next();
+            if (local_tt.getType() == tt_word)
+            {
+              num_consumed_words++;
             }
           }
-        } else {
+        }
+        else
+        {
+          //Add rule to banned rules
           banned_rules.insert(lastrule_id);
           input_buffer.setPos(prev_last);
           input_buffer.next();
@@ -852,11 +867,11 @@ Postchunk::unchunk(UString const &chunk, UFILE* output)
   bool uppercase_all = false;
   bool uppercase_first = false;
 
-  if(case_info == "AA"_u)
+  if(!dictionary_case && case_info == "AA"_u)
   {
     uppercase_all = true;
   }
-  else if(case_info == "Aa"_u)
+  else if(!dictionary_case && case_info == "Aa"_u)
   {
     uppercase_first = true;
   }
@@ -987,15 +1002,13 @@ Postchunk::splitWordsAndBlanks(UString const &chunk, vector<UString *> &words,
           if(u_isdigit(chunk[i+1]))
           {
             // replace tag
-            unsigned long value = StringUtils::stoi(chunk.c_str()+i+1) - 1;
-            // TODO
-            //unsigned long value = wcstoul(chunk.c_str()+i+1,
-            //                              NULL, 0) - 1;
+            int j = ++i;
+            while (chunk[++i] != '>');
+            unsigned long value = StringUtils::stoi(chunk.substr(j, i-j)) - 1;
             if(vectags.size() > value)
             {
               ref.append(vectags[value]);
             }
-            while(chunk[++i] != '>');
           }
           else
           {
@@ -1078,15 +1091,13 @@ Postchunk::splitWordsAndBlanks(UString const &chunk, vector<UString *> &words,
             if(u_isdigit(chunk[i+1]))
             {
               // replace tag
-              unsigned long value = StringUtils::stoi(chunk.c_str()+i+1) - 1;
-              //unsigned long value = wcstoul(chunk.c_str()+i+1,
-              //                              NULL, 0) - 1;
-              // TODO: make sure this is equivalent
+              int j = ++i;
+              while (chunk[++i] != '>');
+              unsigned long value = StringUtils::stoi(chunk.substr(j, i-j)) - 1;
               if(vectags.size() > value)
               {
                 ref.append(vectags[value]);
               }
-              while(chunk[++i] != '>');
             }
             else
             {
