@@ -24,9 +24,6 @@
 using namespace std;
 
 Transfer::Transfer()
-  : word(nullptr), last_lword(0), in_lu(false), in_wblank(false),
-    isExtended(false), defaultAttrs(lu), preBilingual(false),
-    useBilingual(true), trace_att(false)
 {}
 
 void
@@ -436,7 +433,7 @@ Transfer::processChunk(xmlNode *localroot)
   }
 
   result += '^';
-  if(!caseofchunk.empty())
+  if(!caseofchunk.empty() && !dictionary_case)
   {
     if(!name.empty())
     {
@@ -656,6 +653,7 @@ Transfer::processLet(xmlNode *localroot)
 void
 Transfer::processModifyCase(xmlNode *localroot)
 {
+  if (dictionary_case) return;
   xmlNode *leftSide = NULL, *rightSide = NULL;
 
   for (auto i : children(localroot)) {
@@ -863,6 +861,15 @@ Transfer::readToken(InputFile& in)
           content += ']';
           break;
         }
+        else if(val2 == U_EOF)
+        {
+          break;                // should not happen
+        }
+        else if(val2 == '\0')
+        {
+          in.unget(val2);
+          break;
+        }
         else
         {
           content += val2;
@@ -999,7 +1006,7 @@ Transfer::transfer(InputFile& in, UFILE* out)
             input_buffer.setPos(prev_last+1);
           }
           int num_consumed_words = 0;
-          while(num_consumed_words < num_words_to_consume)
+          while(num_consumed_words < num_words_to_consume && !input_buffer.isEmpty())
           {
             TransferToken& local_tt = input_buffer.next();
             if (local_tt.getType() == tt_word)
@@ -1051,30 +1058,15 @@ Transfer::transfer(InputFile& in, UFILE* out)
             UString tl;
             UString ref;
             UString wblank;
+            UString cur;
 
             int seenSlash = 0;
             for(UString::const_iterator it = tmpword[0]->begin(); it != tmpword[0]->end(); it++)
             {
-              if(*it == '\\')
-              {
-                if(seenSlash == 0)
-                {
-                  sl.push_back(*it);
-                  it++;
-                  sl.push_back(*it);
-                }
-                else if(seenSlash == 1)
-                {
-                  tl.push_back(*it);
-                  it++;
-                  tl.push_back(*it);
-                }
-                else
-                {
-                  ref.push_back(*it);
-                  it++;
-                  ref.push_back(*it);
-                }
+              if(*it == '\\') {
+                cur.push_back(*it);
+                it++;
+                cur.push_back(*it);
                 continue;
               }
               else if(*it == '[')
@@ -1103,39 +1095,39 @@ Transfer::transfer(InputFile& in, UFILE* out)
                 }
                 else
                 {
-                  if(seenSlash == 0)
-                  {
-                    sl.push_back(*it);
-                  }
-                  else if(seenSlash == 1)
-                  {
-                    tl.push_back(*it);
-                  }
-                  else
-                  {
-                    ref.push_back(*it);
-                  }
+                  cur.push_back(*it);
                 }
                 continue;
               }
               else if(*it == '/')
               {
+                ref.clear();
+                switch (seenSlash) {
+                case 0: cur.swap(sl); break;
+                case 1: cur.swap(tl); break;
+                default: cur.swap(ref);
+                }
                 seenSlash++;
-
-                ref.clear(); //the word after the last slash is the ref
                 continue;
               }
-              if(seenSlash == 0)
-              {
-                sl.push_back(*it);
+              else if (*it == '<') {
+                while (*it != '>') {
+                  cur.push_back(*it);
+                  if (*it == '\\') {
+                    it++;
+                    cur.push_back(*it);
+                  }
+                  it++;
+                }
               }
-              else if(seenSlash == 1)
-              {
-                tl.push_back(*it);
-              }
-              else
-              {
-                ref.push_back(*it);
+              cur.push_back(*it);
+            }
+            if (!cur.empty()) {
+              ref.clear();
+              switch (seenSlash) {
+              case 0: cur.swap(sl); break;
+              case 1: cur.swap(tl); break;
+              default: cur.swap(ref);
               }
             }
             //tmpword[0]->assign(sl);
@@ -1259,20 +1251,18 @@ Transfer::applyRule()
       }
     }
 
-    pair<UString, int> tr;
-    if(useBilingual && preBilingual == false)
-    {
-      tr = fstp.biltransWithQueue(*tmpword[i], false);
+    if(useBilingual && preBilingual == false) {
+      auto tr = fstp.biltransWithQueue(*tmpword[i], false);
       word[i] = new TransferWord(*tmpword[i], tr.first, ""_u, ""_u, tr.second);
     }
-    else if(preBilingual)
-    {
+    else if(preBilingual) {
       UString sl;
       UString tl;
       UString ref;
       UString wblank;
 
       int seenSlash = 0;
+      bool inTag = false;
       for(UString::const_iterator it = tmpword[i]->begin(); it != tmpword[i]->end(); it++)
       {
         if(*it == '\\')
@@ -1301,71 +1291,53 @@ Transfer::applyRule()
         {
           if(*(it+1) == '[') //wordbound blank
           {
-            while(true)
-            {
-              if(*it == '\\')
-              {
+            while(true) {
+              if(*it == '\\') {
                 wblank.push_back(*it);
                 it++;
                 wblank.push_back(*it);
               }
-              else if(*it == '^' && *(it-1) == ']' && *(it-2) == ']')
-              {
+              else if(*it == '^' && *(it-1) == ']' && *(it-2) == ']') {
                 break;
               }
-              else
-              {
+              else {
                 wblank.push_back(*it);
               }
 
               it++;
             }
           }
-          else
-          {
-            if(seenSlash == 0)
-            {
-              sl.push_back(*it);
-            }
-            else if(seenSlash == 1)
-            {
-              tl.push_back(*it);
-            }
-            else
-            {
-              ref.push_back(*it);
-            }
+          else {
+            if(seenSlash == 0)      sl.push_back(*it);
+            else if(seenSlash == 1) tl.push_back(*it);
+            else                   ref.push_back(*it);
           }
           continue;
         }
 
-        if(*it == '/')
-        {
+        if(*it == '/' && !inTag) {
           seenSlash++;
 
           ref.clear(); //word after last slash is ref
           continue;
         }
-        if(seenSlash == 0)
-        {
-          sl.push_back(*it);
+        // slashes in tags are accepted elsewhere in the pipe,
+        // so don't split readings on them here
+        else if (*it == '<') {
+          inTag = true;
         }
-        else if(seenSlash == 1)
-        {
-          tl.push_back(*it);
-        }
-        else
-        {
-          ref.push_back(*it);
+        else if (*it == '>') {
+          inTag = false;
         }
+        if(seenSlash == 0)      sl.push_back(*it);
+        else if(seenSlash == 1) tl.push_back(*it);
+        else                   ref.push_back(*it);
       }
-      tr = pair<UString, int>(tl, false);
-      word[i] = new TransferWord(sl, tr.first, ref, wblank, tr.second);
+      word[i] = new TransferWord(sl, tl, ref, wblank, false);
     }
     else // neither useBilingual nor preBilingual (sl==tl)
     {
-      tr = pair<UString, int>(*tmpword[i], false);
-      word[i] = new TransferWord(*tmpword[i], tr.first, ""_u, ""_u, tr.second);
+      word[i] = new TransferWord(*tmpword[i], *tmpword[i], ""_u, ""_u, false);
     }
   }
 
