@@ -12,12 +12,11 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 #include <apertium/transfer_instr.h>
-#include <apertium/string_utils.h>
+#include <lttoolbox/string_utils.h>
 
-using namespace Apertium;
 void
 TransferInstr::copy(TransferInstr const &o)
 {
@@ -26,6 +25,7 @@ TransferInstr::copy(TransferInstr const &o)
   pos = o.pos;
   pointer = o.pointer;
   condition = o.condition;
+  strval = o.strval;
 }
 
 void
@@ -33,14 +33,16 @@ TransferInstr::destroy()
 {
 }
 
-TransferInstr::TransferInstr(TransferInstrType t, string const &c,
-                             int const p, void *ptr, bool cond)
+TransferInstr::TransferInstr(TransferInstrType t, UString const &c,
+                             int const p, xmlNode* ptr, bool cond,
+                             const UString& sv)
 {
   type = t;
   content = c;
   pos = p;
   pointer = ptr;
   condition = cond;
+  strval = sv;
 }
 
 TransferInstr::~TransferInstr()
@@ -70,7 +72,7 @@ TransferInstr::getType()
   return type;
 }
 
-string const &
+UString const &
 TransferInstr::getContent()
 {
   return content;
@@ -82,7 +84,7 @@ TransferInstr::getPos()
   return pos;
 }
 
-void *
+xmlNode*
 TransferInstr::getPointer()
 {
   return pointer;
@@ -93,3 +95,9 @@ TransferInstr::getCondition()
 {
   return condition;
 }
+
+const UString&
+TransferInstr::getStrval()
+{
+  return strval;
+}
